@@ -7,7 +7,9 @@
 #' \code{\link{msaenet}}, \code{\link{msamnet}}, or \code{\link{msasnet}}.
 #' @param type Plot type, \code{"coef"} for a coefficient path plot
 #' across all estimation steps; \code{"criterion"} for a scree plot of
-#' the model evaluation criterion used (CV error, AIC, BIC, or EBIC).
+#' the model evaluation criterion used (CV error, AIC, BIC, or EBIC);
+#' \code{"dotplot"} for a Cleveland dot plot of the coefficients
+#' estimated by the model at the optimal step.
 #' @param nsteps Maximum number of estimation steps to plot.
 #' Default is to plot all steps.
 #' @param highlight Should we highlight the "optimal" step
@@ -15,9 +17,9 @@
 #' @param col Color palette to use for the coefficient paths.
 #' If it is \code{NULL}, a default color palette will be assigned.
 #' @param label Should we label all the non-zero variables of the
-#' optimal step in the coefficient plot? Default is \code{FALSE}.
-#' If \code{TRUE} and \code{label.vars = NULL}, the index of the
-#' variables will be used as labels.
+#' optimal step in the coefficient plot or the dot plot?
+#' Default is \code{FALSE}. If \code{TRUE} and \code{label.vars = NULL},
+#' the index of the non-zero variables will be used as labels.
 #' @param label.vars Labels to use for all the variables
 #' if \code{label = "TRUE"}.
 #' @param label.pos Position of the labels. See argument
@@ -26,15 +28,22 @@
 #' \code{offset} in \code{\link[graphics]{text}} for details.
 #' @param label.cex Character expansion factor of the labels.
 #' See argument \code{cex} in \code{\link[graphics]{text}} for details.
+#' @param label.srt Label rotation in degrees for the Cleveland dot plot.
+#' Default is \code{90}. See argument \code{srt} in
+#' \code{\link[graphics]{par}} for details.
 #' @param xlab Title for x axis. If is \code{NULL}, will use the default title.
 #' @param ylab Title for y axis. If is \code{NULL}, will use the default title.
+#' @param abs Should we plot the absolute values of the coefficients
+#' instead of the raw coefficients in the Cleveland dot plot?
+#' Default is \code{FALSE}.
 #' @param ... Other parameters (not used).
 #'
 #' @method plot msaenet
 #'
 #' @author Nan Xiao <\url{https://nanx.me}>
 #'
-#' @importFrom graphics axis lines matlines matplot plot text
+#' @importFrom graphics axis lines matlines matplot plot text abline points
+#' @importFrom stats coef
 #'
 #' @export
 #'
@@ -55,13 +64,17 @@
 #' plot(msasnet.fit, label = TRUE, nsteps = 5)
 #' plot(msasnet.fit, type = "criterion")
 #' plot(msasnet.fit, type = "criterion", nsteps = 5)
+#' plot(msasnet.fit, type = "dotplot", label = TRUE)
+#' plot(msasnet.fit, type = "dotplot", label = TRUE, abs = TRUE)
 
 plot.msaenet = function(
-  x, type = c('coef', 'criterion'), nsteps = NULL,
+  x, type = c('coef', 'criterion', 'dotplot'), nsteps = NULL,
   highlight = TRUE, col = NULL,
   label = FALSE, label.vars = NULL,
   label.pos = 2, label.offset = 0.3, label.cex = 0.7,
-  xlab = NULL, ylab = NULL, ...) {
+  label.srt = 90,
+  xlab = NULL, ylab = NULL,
+  abs = FALSE, ...) {
 
   type = match.arg(type)
 
@@ -100,6 +113,13 @@ plot.msaenet = function(
                     xlab, ylab)
   }
 
+  if (type == 'dotplot') {
+    .dotplot(
+      x, abs,
+      label, label.vars, label.cex, label.srt,
+      xlab, ylab)
+  }
+
   invisible()
 
 }
@@ -186,3 +206,57 @@ plot.msaenet = function(
     col = 'darkred', lty = 2, lwd = 1.5)
 
 }
+
+# Cleveland dot plot for model coefficients at the optimal step
+.dotplot = function(
+  x, abs, label, label.vars, label.cex, label.srt, xlab, ylab) {
+
+  idx.nzv = msaenet.nzv(x)
+
+  if (is.null(xlab)) xlab = 'Selected Variables'
+  if (is.null(ylab)) ylab = 'Coefficients'
+
+  if (label & is.null(label.vars)) {
+    label.nzv = as.character(idx.nzv)
+  } else if (label & !is.null(label.vars)) {
+    label.nzv = label.vars[idx.nzv]
+  } else {
+    label.nzv = rep('', length(idx.nzv))
+  }
+
+  coef.nzv = coef(x)[idx.nzv]
+  if (abs) coef.nzv = abs(coef.nzv)
+
+  ord.nzv = order(coef.nzv, decreasing = TRUE)
+
+  if (all(coef.nzv > 0)) {
+    plot(coef.nzv[ord.nzv], type = 'h', xaxt = 'n', xlab = xlab, ylab = ylab,
+         ylim = c(-0.3, max(coef.nzv) + 0.05))
+  } else if (all(coef.nzv < 0)) {
+    plot(coef.nzv[ord.nzv], type = 'h', xaxt = 'n', xlab = xlab, ylab = ylab,
+         ylim = c(min(coef.nzv) - 0.05, 0.3))
+  } else {
+    plot(coef.nzv[ord.nzv], type = 'h', xaxt = 'n', xlab = xlab, ylab = ylab)
+  }
+
+  abline(a = 0, b = 0)
+
+  col.vec = ifelse(coef.nzv > 0, '#BC3C29', '#0072B5')
+  points(
+    1L:length(coef.nzv), coef.nzv[ord.nzv], pch = 21,
+    bg = col.vec[ord.nzv], col = col.vec[ord.nzv])
+
+  idx.pos = which(coef.nzv[ord.nzv] > 0)
+  idx.neg = which(coef.nzv[ord.nzv] <= 0)
+
+  if (length(idx.pos) > 0L)
+    text(
+      idx.pos, -0.05, labels = label.nzv[ord.nzv][idx.pos],
+      srt = label.srt, cex = label.cex, adj = 1)
+
+  if (length(idx.neg) > 0L)
+    text(
+      idx.neg, 0.05, labels = label.nzv[ord.nzv][idx.neg],
+      srt = label.srt, cex = label.cex, adj = 0)
+
+}
